@@ -1,12 +1,19 @@
+
 ##	common/xmpp/socks5.py
 ##
-## Gajim Team:
+## Contributors for this file:
 ##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Vincent Hanquez <tab@snarc.org>
 ##	- Nikos Kouremenos <nkour@jabber.org>
 ##	- Dimitur Kirov <dkirov@gmail.com>
 ##
-##	Copyright (C) 2003-2005 Gajim Team
+## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
+##                         Vincent Hanquez <tab@snarc.org>
+## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
+##                    Vincent Hanquez <tab@snarc.org>
+##                    Nikos Kouremenos <nkour@jabber.org>
+##                    Dimitur Kirov <dkirov@gmail.com>
+##                    Travis Shirk <travis@pobox.com>
+##                    Norman Rasmussen <norman@rasmussen.co.za>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -24,6 +31,7 @@ import select
 import os
 import struct
 import sha
+import time
 
 from errno import EWOULDBLOCK
 from errno import ENOBUFS
@@ -33,8 +41,7 @@ MAX_BUFF_LEN = 65536
 
 class SocksQueue:
 	''' queue for all file requests objects '''
-	def __init__(self, complete_transfer_cb = None, 
-		progress_transfer_cb = None):
+	def __init__(self, complete_transfer_cb = None, progress_transfer_cb = None):
 		self.connected = 0
 		self.readers = {}
 		self.files_props = {}
@@ -118,7 +125,7 @@ running instance of Gajim. \nFile Transfer will be canceled.\n==================
 			for host in file_props['streamhosts']:
 				if host['state'] != -1:
 					return
-		if file_props['failure_cb']:
+		if file_props.has_key('failure_cb') and file_props['failure_cb']:
 			file_props['failure_cb'](streamhost['initiator'], streamhost['id'], 
 				file_props['sid'], code = 404)
 		
@@ -169,9 +176,10 @@ running instance of Gajim. \nFile Transfer will be canceled.\n==================
 			reader.file_props['completed'] = False
 			reader.file_props['paused'] = False
 			reader.file_props['stalled'] = False
+			reader.file_props['elapsed-time'] = 0
+			reader.file_props['last-time'] = time.time()
 			reader.file_props['received-len'] = 0
 			reader.pauses = 0
-			reader.send_raw(reader._get_nl_byte())
 			result = reader.write_next()
 			self.process_result(result, reader)
 	
@@ -186,6 +194,8 @@ running instance of Gajim. \nFile Transfer will be canceled.\n==================
 				result = sender.send_file()
 				self.process_result(result, sender)
 			else:
+				file_props['elapsed-time'] = 0
+				file_props['last-time'] = time.time()
 				file_props['received-len'] = 0
 				sender.file_props = file_props
 				
@@ -248,17 +258,20 @@ running instance of Gajim. \nFile Transfer will be canceled.\n==================
 					sender.file_props['type'] == 'r':
 						result = sender.get_file_contents(0)
 						self.process_result(result, sender)
-				elif sender.state == 7 and not sender.file_props['paused']:
-					if not sender.connected:
-						self.process_result(-1, sender)
-						break
-					if sender.state == 8:
-						self.remove_sender(idx)
-						break
-					result = sender.write_next()
-					self.process_result(result, sender)
-					if sender.file_props['stalled']:
-						break
+				elif sender.state == 7:
+					while True:
+						if sender.file_props['paused']:
+							break
+						if not sender.connected:
+							self.process_result(-1, sender)
+							break
+						if sender.state == 8:
+							self.remove_sender(idx)
+							break
+						result = sender.write_next()
+						self.process_result(result, sender)
+						if result is None or result <= 0:
+							break
 				elif sender.state == 8:
 					self.remove_sender(idx)
 			else:
@@ -298,14 +311,11 @@ running instance of Gajim. \nFile Transfer will be canceled.\n==================
 		'''
 		if result is None:
 			return
-		if result in [0, -1] and \
-			self.complete_transfer_cb is not None:
+		if result in (0, -1) and self.complete_transfer_cb is not None:
 			account = actor.account
-			self.complete_transfer_cb(account, 
-				actor.file_props)
+			self.complete_transfer_cb(account, actor.file_props)
 		elif self.progress_transfer_cb is not None:
-			self.progress_transfer_cb(actor.account, 
-				actor.file_props)
+			self.progress_transfer_cb(actor.account, actor.file_props)
 	
 	def remove_receiver(self, idx, do_disconnect = True):
 		''' Remove reciver from the list and decrease 
@@ -350,10 +360,16 @@ class Socks5:
 		self.pauses = 0
 		self.size = 0
 		self.remaining_buff = ''
+		self.fd = None
 		
 	def open_file_for_reading(self):
-		self.fd = open(self.file_props['file-name'],'rb')
-		self.fd.seek(self.size)
+		if self.fd == None:
+			try:
+				self.fd = open(self.file_props['file-name'],'rb')
+			except IOError, e:
+				self.close_file()
+				raise IOError, e
+		
 	def close_file(self):
 		try:
 			self.fd.close()
@@ -369,6 +385,8 @@ class Socks5:
 		else:
 			fd = open(self.file_props['file-name'],'wb')
 			self.file_props['fd'] = fd
+			self.file_props['elapsed-time'] = 0
+			self.file_props['last-time'] = time.time()
 			self.file_props['received-len'] = 0
 		return fd
 
@@ -405,36 +423,25 @@ class Socks5:
 			self.disconnect()
 		return len(raw_data)
 	
-	def write_next(self, initial = False):
-		if initial:
-			try:
-				# send
-				lenn = self._send(self._get_nl_byte())
-			except Exception, e:
-				errnum = e.args[0]
-				# peer cannot read at the moment, continue later
-				if errnum in [EINTR, ENOBUFS, EWOULDBLOCK]:
-					pass
-				else: # peer will not be able to read anymore
-					self.state = 8
-					self.disconnect()
-					self.file_props['error'] = -1
-					return -1
-			self.state = 7
-			return 1
+	def write_next(self):
 		if self.remaining_buff != '':
 			buff = self.remaining_buff
 			self.remaining_buff = ''
 		else:
-			self.open_file_for_reading()
+			try:
+				self.open_file_for_reading()
+			except IOError, e:
+				self.state = 8 # end connection
+				self.disconnect()
+				self.file_props['error'] = -7 # unable to read from file
+				return -1
 			buff = self.fd.read(MAX_BUFF_LEN)
-			self.close_file()
 		if len(buff) > 0:
 			lenn = 0
 			try:
 				lenn = self._send(buff)
 			except Exception, e:
-				if e.args[0] not in [EINTR, ENOBUFS, EWOULDBLOCK]:
+				if e.args[0] not in (EINTR, ENOBUFS, EWOULDBLOCK):
 					# peer stopped reading
 					self.state = 8 # end connection
 					self.close_file()
@@ -442,10 +449,15 @@ class Socks5:
 					self.file_props['error'] = -1
 					return -1
 			self.size += lenn
+			current_time = time.time()
+			self.file_props['elapsed-time'] += current_time - \
+				self.file_props['last-time']
+			self.file_props['last-time'] = current_time
 			self.file_props['received-len'] = self.size
-			if self.size == int(self.file_props['size']):
+			if self.size >= int(self.file_props['size']):
 				self.state = 8 # end connection
 				self.file_props['error'] = 0
+				self.close_file()
 				self.disconnect()
 				return -1
 			if lenn != len(buff):
@@ -465,9 +477,8 @@ class Socks5:
 				return None
 			return lenn
 		else:
-			return 0
 			self.state = 8 # end connection
-			
+			self.close_file()
 			self.disconnect()
 			return -1
 	
@@ -483,6 +494,10 @@ class Socks5:
 			fd = self.get_fd()
 			fd.write(self.remaining_buff)
 			lenn = len(self.remaining_buff)
+			current_time = time.time()
+			self.file_props['elapsed-time'] += current_time - \
+				self.file_props['last-time']
+			self.file_props['last-time'] = current_time
 			self.file_props['received-len'] += lenn
 			self.remaining_buff = ''
 			if self.file_props['received-len'] == int(self.file_props['size']):
@@ -499,14 +514,24 @@ class Socks5:
 				except Exception, e:
 					buff = ''
 				first_byte = False
-				if self.file_props['received-len'] == 0:
-					if len(buff) > 0:
-						# delimiter between auth and data
-						if ord(buff[0]) == 0xD:
-							first_byte = True
+				if self.file_props['received-len'] == 0:  
+					if len(buff) > 0:  
+						# delimiter between auth and data  
+						if ord(buff[0]) == 0xD:  
+							first_byte = True  
 							buff = buff[1:]
+				current_time = time.time()
+				self.file_props['elapsed-time'] += current_time - \
+					self.file_props['last-time']
+				self.file_props['last-time'] = current_time
 				self.file_props['received-len'] += len(buff)
-				fd.write(buff)
+				try:
+					fd.write(buff)
+				except IOError, e:
+					self.rem_fd(fd)
+					self.disconnect(False)
+					self.file_props['error'] = -6 # file system error
+					return 0
 				if len(buff) == 0 and first_byte is False:
 					# Transfer stopped  somehow:
 					# reset, paused or network error
@@ -514,7 +539,7 @@ class Socks5:
 					self.disconnect(False)
 					self.file_props['error'] = -1
 					return 0
-				if self.file_props['received-len'] == int(self.file_props['size']):
+				if self.file_props['received-len'] >= int(self.file_props['size']):
 					# transfer completed
 					self.rem_fd(fd)
 					self.disconnect()
@@ -571,7 +596,7 @@ class Socks5:
 		auth_mechanisms = []
 		try:
 			ver, num_auth = struct.unpack('!BB', buff[:2])
-			for i in range(num_auth):
+			for i in xrange(num_auth):
 				mechanism, = struct.unpack('!B', buff[1 + i])
 				auth_mechanisms.append(mechanism)
 		except:
@@ -596,10 +621,6 @@ class Socks5:
 		buff = struct.pack('!BBBBB%dsBB' % len(msg), 
 			0x05, command, 0x00, 0x03, len(msg), msg, 0, 0)
 		return buff
-	
-	def _get_nl_byte(self):
-		''' This is sent between auth and real data '''
-		return struct.pack('!B', 0x0D)
 		
 	def _parse_request_buff(self, buff):
 		try: # don't trust on what comes from the outside
@@ -659,7 +680,6 @@ class Socks5Sender(Socks5):
 		
 	def send_file(self):
 		''' start sending the file over verified connection ''' 
-		self.open_file_for_reading()
 		self.file_props['error'] = 0
 		self.file_props['disconnect_cb'] = self.disconnect
 		self.file_props['started'] = True
@@ -667,9 +687,12 @@ class Socks5Sender(Socks5):
 		self.file_props['paused'] = False
 		self.file_props['stalled'] = False
 		self.file_props['connected'] = True
+		self.file_props['elapsed-time'] = 0
+		self.file_props['last-time'] = time.time()
 		self.file_props['received-len'] = 0
 		self.pauses = 0
-		return self.write_next(initial = True) # initial for nl byte
+		self.state = 7
+		return self.write_next() # initial for nl byte
 		
 	def main(self):
 		''' initial requests for verifying the connection '''
@@ -692,14 +715,14 @@ class Socks5Sender(Socks5):
 		self.state += 1 # go to the next step
 		return None
 			
-	def pending_data(self,timeout=0.01):
+	def pending_data(self,timeout=0):
 		''' return true if there is a data ready to be read '''
 		if self._sock is None:
 			return False
 		try:
-			if self.state in [1, 3, 5]:  # (initial, request, send file)
+			if self.state in (1, 3, 5):  # (initial, request, send file)
 				return self.pending_read(timeout)
-			elif self.state in [2, 4]: # send auth and positive response
+			elif self.state in (2, 4): # send auth and positive response
 				return True
 		except Exception, e:
 			return False
@@ -752,7 +775,7 @@ class Socks5Listener:
 		_sock[0].setblocking(False)
 		return _sock
 	
-	def pending_connection(self,timeout=0.005):
+	def pending_connection(self,timeout=0):
 		''' Returns true if there is a data ready to be read. '''
 		if self._serv is None:
 			return False
@@ -872,9 +895,10 @@ class Socks5Receiver(Socks5):
 				self.file_props['completed'] = False
 				self.file_props['paused'] = False
 				self.file_props['stalled'] = False
+				self.file_props['elapsed-time'] = 0
+				self.file_props['last-time'] = time.time()
 				self.file_props['received-len'] = 0
 				self.pauses = 0
-				self.send_raw(self._get_nl_byte())
 			self.state = 6 # send/get file contents
 		if self.state < 5:
 			self.state += 1
@@ -886,9 +910,9 @@ class Socks5Receiver(Socks5):
 		if self._sock is None:
 			return False
 		try:
-			if self.state in [2, 4, 6]: # auth response, connect, file data
-				return self.pending_read(0.01)
-			elif self.state in [1, 3, 5]: # auth types, connect request
+			if self.state in (2, 4, 6): # auth response, connect, file data
+				return self.pending_read(0)
+			elif self.state in (1, 3, 5): # auth types, connect request
 				return True
 		except Exception, e:
 			return False
