@@ -1,12 +1,18 @@
 ##	filetransfers_window.py
 ##
-## Gajim Team:
+## Contributors for this file:
 ##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Vincent Hanquez <tab@snarc.org>
 ##	- Nikos Kouremenos <kourem@gmail.com>
 ##	- Dimitur Kirov <dkirov@gmail.com>
 ##
-##	Copyright (C) 2003-2005 Gajim Team
+## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
+##                         Vincent Hanquez <tab@snarc.org>
+## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
+##                    Vincent Hanquez <tab@snarc.org>
+##                    Nikos Kouremenos <nkour@jabber.org>
+##                    Dimitur Kirov <dkirov@gmail.com>
+##                    Travis Shirk <travis@pobox.com>
+##                    Norman Rasmussen <norman@rasmussen.co.za>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -21,8 +27,10 @@
 import gtk
 import gtk.glade
 import gobject
+import pango
 import os
 import sys
+import time
 
 import gtkgui_helpers
 import tooltips
@@ -39,17 +47,25 @@ gtk.glade.textdomain (APP)
 
 GTKGUI_GLADE = 'gtkgui.glade'
 
+C_IMAGE = 0
+C_LABELS = 1
+C_FILE = 2
+C_TIME = 3
+C_PROGRESS = 4
+C_PERCENT = 5
+C_SID = 6
+
+
 class FileTransfersWindow:
-	def __init__(self, plugin):
+	def __init__(self):
 		self.files_props = {'r' : {}, 's': {}}
-		self.plugin = plugin
 		self.height_diff = 0
 		self.xml = gtk.glade.XML(GTKGUI_GLADE, 'file_transfers_window', APP)
 		self.window = self.xml.get_widget('file_transfers_window')
 		self.tree = self.xml.get_widget('transfers_list')
 		self.cancel_button = self.xml.get_widget('cancel_button')
 		self.pause_button = self.xml.get_widget('pause_restore_button')
-		self.remove_button = self.xml.get_widget('remove_button')
+		self.cleanup_button = self.xml.get_widget('cleanup_button')
 		self.notify_ft_checkbox = self.xml.get_widget(
 			'notify_ft_complete_checkbox')
 		notify = gajim.config.get('notify_on_file_complete')
@@ -57,7 +73,7 @@ class FileTransfersWindow:
 			self.notify_ft_checkbox.set_active(True)
 		else:
 			self.notify_ft_checkbox.set_active(False)
-		self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, str, str, str, str)
+		self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, str, str, str, int, str)
 		self.tree.set_model(self.model)
 		col = gtk.TreeViewColumn()
 		
@@ -73,25 +89,41 @@ class FileTransfersWindow:
 		col = gtk.TreeViewColumn(_('File'))
 		renderer = gtk.CellRendererText()
 		col.pack_start(renderer, expand=False)
-		col.add_attribute(renderer, 'markup' , 1)
+		col.add_attribute(renderer, 'markup' , C_LABELS)
 		renderer.set_property('yalign', 0.)
 		renderer = gtk.CellRendererText()
 		col.pack_start(renderer, expand=True)
-		col.add_attribute(renderer, 'markup' , 2)
+		col.add_attribute(renderer, 'markup' , C_FILE)
 		renderer.set_property('xalign', 0.)
 		renderer.set_property('yalign', 0.)
+		renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
 		col.set_resizable(True)
 		col.set_expand(True)
 		self.tree.append_column(col)
 		
-		col = gtk.TreeViewColumn(_('Progress'))
+		col = gtk.TreeViewColumn(_('Time'))
 		renderer = gtk.CellRendererText()
-		renderer.set_property('yalign', 0.)
-		renderer.set_property('xalign', 0.)
-		col.pack_start(renderer, expand = True)
+		col.pack_start(renderer, expand=False)
+		col.add_attribute(renderer, 'markup' , C_TIME)
+		renderer.set_property('yalign', 0.5)
+		renderer.set_property('xalign', 0.5)
+		renderer = gtk.CellRendererText()
+		renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
+		col.set_resizable(True)
 		col.set_expand(False)
-		col.add_attribute(renderer, 'text' , 3)
 		self.tree.append_column(col)
+		
+		col = gtk.TreeViewColumn(_('Progress'))
+		renderer = gtk.CellRendererProgress()
+		renderer.set_property('yalign', 0.5)
+		renderer.set_property('xalign', 0.5)
+		col.pack_start(renderer, expand = False)
+		col.add_attribute(renderer, 'text' , C_PROGRESS)
+		col.add_attribute(renderer, 'value' , C_PERCENT)
+		col.set_resizable(True)
+		col.set_expand(False)
+		self.tree.append_column(col)
+		
 		self.set_images()
 		self.tree.get_selection().set_mode(gtk.SELECTION_SINGLE)
 		self.tree.get_selection().connect('changed', self.selection_changed)
@@ -104,11 +136,9 @@ class FileTransfersWindow:
 		self.cancel_menuitem = popup_xml.get_widget('cancel_menuitem')
 		self.pause_menuitem = popup_xml.get_widget('pause_menuitem')
 		self.continue_menuitem = popup_xml.get_widget('continue_menuitem')
+		self.continue_menuitem.hide()
+		self.continue_menuitem.set_no_show_all(True)
 		self.remove_menuitem = popup_xml.get_widget('remove_menuitem')
-		self.clean_up_menuitem = popup_xml.get_widget('clean_up_menuitem')
-		if gtk.gtk_version >= (2, 6, 0) and gtk.pygtk_version >= (2, 6, 0):
-			self.pause_button.set_image(gtk.image_new_from_stock(
-		gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_MENU))
 		popup_xml.signal_autoconnect(self)
 		
 	def find_transfer_by_jid(self, account, jid):
@@ -151,7 +181,7 @@ class FileTransfersWindow:
 				file_props['tt_account'], jid).name
 			sender = gtkgui_helpers.escape_for_pango_markup(sender_name)
 		else:
-			#You is a reply of who send a file
+			#You is a reply of who sent a file
 			sender = _('You')
 		sectext += '\n\t' +_('Sender: %s') % sender
 		sectext += '\n\t' +_('Recipient: ')
@@ -162,22 +192,17 @@ class FileTransfersWindow:
 			recipient = gtkgui_helpers.escape_for_pango_markup(receiver_name)
 		else:
 			#You is a reply of who received a file
-			recipient = ('You')
+			recipient = _('You')
 		sectext += recipient
 		if file_props['type'] == 'r':
 			sectext += '\n\t' +_('Saved in: %s') % \
 				gtkgui_helpers.escape_for_pango_markup(file_path)
-		dialog = dialogs.HigDialog(None, _('File transfer completed'), sectext, 
-			gtk.STOCK_DIALOG_INFO, [
-				[_('_Open Containing Folder'), gtk.RESPONSE_ACCEPT], 
-				[ gtk.STOCK_OK, gtk.RESPONSE_OK ]])
-		button = dialog.get_button(1)
-		if gtk.gtk_version >= (2, 6, 0) and gtk.pygtk_version >= (2, 6, 0):
-			button.set_image(gtk.image_new_from_stock(
-		gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_BUTTON))
+		dialog = dialogs.HigDialog(None, gtk.MESSAGE_INFO, gtk.BUTTONS_NONE, 
+				_('File transfer completed'), sectext)
+		if file_props['type'] == 'r':
+			dialog.add_buttons(_('_Open Containing Folder'), gtk.RESPONSE_ACCEPT)
+		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK)
 		dialog.show_all()
-		if file_props['type'] == 's':
-			button.hide()
 		response = dialog.run()
 		dialog.destroy()
 		if response == gtk.RESPONSE_ACCEPT:
@@ -221,8 +246,6 @@ _('Connection with peer cannot be established.'))
 		self.tree.get_selection().unselect_all()
 		
 	def show_file_send_request(self, account, contact):
-		#FIXME: user better name for this function
-		#atm it's like it shows popup for incoming file transfer request
 		last_send_dir = gajim.config.get('last_send_dir')
 		dialog = gtk.FileChooserDialog(title=_('Choose File to Send...'), 
 			action=gtk.FILE_CHOOSER_ACTION_OPEN, 
@@ -237,33 +260,44 @@ _('Connection with peer cannot be established.'))
 			home_dir = os.path.expanduser('~')
 			dialog.set_current_folder(home_dir)
 		file_props = {}
-		response = dialog.run()
-		if response == gtk.RESPONSE_OK:
-			files_path_list = dialog.get_filenames()
-			dialog.destroy()
-			file_dir = None
-			for file_path in files_path_list:
-				file_path = file_path.decode(sys.getfilesystemencoding())
-				if os.path.isfile(file_path):
-					file_dir = os.path.dirname(file_path)
-					self.send_file(account, contact, file_path)
-			if file_dir:
-				gajim.config.set('last_send_dir', file_dir)
-		else:
-			dialog.destroy()
-
+		while True:
+			response = dialog.run()
+			if response == gtk.RESPONSE_OK:
+				file_dir = None
+				files_path_list = dialog.get_filenames()
+				for file_path in files_path_list:
+					file_path = file_path.decode('utf8')
+					if self.send_file(account, contact, file_path) and file_dir is None:
+						file_dir = os.path.dirname(file_path)
+				if file_dir:
+					gajim.config.set('last_send_dir', file_dir)
+					dialog.destroy()
+					break
+			else:
+				dialog.destroy()
+				break
+		
 	def send_file(self, account, contact, file_path):
 		''' start the real transfer(upload) of the file '''
-		if type(contact) == str:
+		if gtkgui_helpers.file_is_locked(file_path):
+			pritext = _('Gajim cannot access this file')
+			sextext = _('This file is being used by another process.')
+			dialogs.ErrorDialog(pritext, sextext).get_response()
+			return
+		
+		if isinstance(contact, str):
 			if contact.find('/') == -1:
 				return
-			(jid, resource) = contact.split("/", 1)
+			(jid, resource) = contact.split('/', 1)
 			contact = gajim.Contact(jid = jid, resource = resource)
 		(file_dir, file_name) = os.path.split(file_path)
 		file_props = self.get_send_file_props(account, contact, 
 				file_path, file_name)
+		if file_props is None:
+			return False
 		self.add_transfer(account, contact, file_props)
 		gajim.connections[account].send_file_request(file_props)
+		return True
 	
 	def show_file_request(self, account, contact, file_props):
 		''' show dialog asking for comfirmation and store location of new
@@ -271,17 +305,14 @@ _('Connection with peer cannot be established.'))
 		if file_props is None or not file_props.has_key('name'):
 			return
 		last_save_dir = gajim.config.get('last_save_dir')
-		sec_text = '\t' + _('File: %s') % \
-			gtkgui_helpers.escape_for_pango_markup(file_props['name'])
+		sec_text = '\t' + _('File: %s') % file_props['name']
 		if file_props.has_key('size'):
 			sec_text += '\n\t' + _('Size: %s') % \
 				helpers.convert_bytes(file_props['size'])
 		if file_props.has_key('mime-type'):
-			sec_text += '\n\t' + _('Type: %s') % \
-				gtkgui_helpers.escape_for_pango_markup(file_props['mime-type'])
+			sec_text += '\n\t' + _('Type: %s') % file_props['mime-type']
 		if file_props.has_key('desc'):
-			sec_text += '\n\t' + _('Description: %s') % \
-				gtkgui_helpers.escape_for_pango_markup(file_props['desc'])
+			sec_text += '\n\t' + _('Description: %s') % file_props['desc']
 		prim_text = _('%s wants to send you a file:') % contact.jid
 		dialog = dialogs.ConfirmationDialog(prim_text, sec_text)
 		if dialog.get_response() == gtk.RESPONSE_OK:
@@ -291,6 +322,10 @@ _('Connection with peer cannot be established.'))
 				gtk.STOCK_SAVE, gtk.RESPONSE_OK))
 			dialog.set_current_name(file_props['name'])
 			dialog.set_default_response(gtk.RESPONSE_OK)
+			gtk28 = False
+			if gtk.gtk_version >= (2, 8, 0) and gtk.pygtk_version >= (2, 8, 0):
+				dialog.props.do_overwrite_confirmation = True
+				gtk28 = True
 			if last_save_dir and os.path.isdir(last_save_dir):
 				dialog.set_current_folder(last_save_dir)
 			else:
@@ -301,10 +336,7 @@ _('Connection with peer cannot be established.'))
 				if response == gtk.RESPONSE_OK:
 					file_path = dialog.get_filename()
 					file_path = file_path.decode('utf-8')
-					if os.path.exists(file_path):
-						#FIXME: pango does not work here.
-						#FIXME: if gtk2.8 do this via signal
-#http://developer.gnome.org/doc/API/2.0/gtk/GtkFileChooser.html#GtkFileChooser--do-overwrite-confirmation
+					if not gtk28 and os.path.exists(file_path):
 						primtext = _('This file already exists')
 						sectext = _('Would you like to overwrite it?')
 						dialog2 = dialogs.ConfirmationDialog(primtext, sectext)
@@ -346,14 +378,61 @@ _('Connection with peer cannot be established.'))
 		iter = self.get_iter_by_sid(typ, sid)
 		if iter is None:
 			return
-		sid = self.model[iter][4].decode('utf-8')
+		sid = self.model[iter][C_SID].decode('utf-8')
 		file_props = self.files_props[sid[0]][sid[1:]]
 		if status == 'stop':
 			file_props['stopped'] = True
 		elif status == 'ok':
 			file_props['completed'] = True
-		self.model.set(iter, 0, self.images[status])
-	
+		self.model.set(iter, C_IMAGE, self.images[status])
+		
+	def _format_percent(self, percent):
+		''' add extra spaces from both sides of the percent, so that
+		progress string has always a fixed size'''
+		_str = '          '
+		if percent != 100.:
+			_str += ' '
+		if percent < 10:
+			_str += ' '
+		_str += unicode(percent) + '%          \n'
+		return _str
+		
+	def _format_time(self, _time):
+		times = { 'hours': 0, 'minutes': 0, 'seconds': 0 }
+		_time = int(_time)
+		times['seconds'] = _time % 60
+		if _time >= 60:
+			_time /= 60
+			times['minutes'] = _time % 60
+			if _time >= 60:
+				times['hours'] = _time / 60
+		
+		#Print remaining time in format 00:00:00
+		#You can change the places of (hours), (minutes), (seconds) -
+		#they are not translatable.
+		return _('%(hours)02.d:%(minutes)02.d:%(seconds)02.d')  % times
+		
+	def _get_eta_and_speed(self, full_size, transfered_size, elapsed_time):
+		if elapsed_time == 0:
+			return 0., 0.
+		speed = round(float(transfered_size) / elapsed_time)
+		if speed == 0.:
+			return 0., 0.
+		remaining_size = full_size - transfered_size
+		eta = remaining_size / speed
+		return eta, speed
+		
+	def _remove_transfer(self, iter, sid, file_props):
+		self.model.remove(iter)
+		if  file_props.has_key('tt_account'):
+			# file transfer is set
+			account = file_props['tt_account']
+			if gajim.connections.has_key(account):
+				# there is a connection to the account
+				gajim.connections[account].remove_transfer(file_props)
+		del(self.files_props[sid[0]][sid[1:]])
+		del(file_props)
+		
 	def set_progress(self, typ, sid, transfered_size, iter = None):
 		''' change the progress of a transfer with new transfered size'''
 		if not self.files_props[typ].has_key(sid):
@@ -367,13 +446,30 @@ _('Connection with peer cannot be established.'))
 		if iter is None:
 			iter = self.get_iter_by_sid(typ, sid)
 		if iter is not None:
-			text = unicode(percent) + '%\n' 
+			text = self._format_percent(percent)
 			if transfered_size == 0:
 				text += '0'
 			else:
 				text += helpers.convert_bytes(transfered_size)
 			text += '/' + helpers.convert_bytes(full_size)
-			self.model.set(iter, 3, text)
+			# Kb/s
+			
+			# remaining time
+			eta, speed = self._get_eta_and_speed(full_size, transfered_size, 
+				file_props['elapsed-time'])
+			
+			self.model.set(iter, C_PROGRESS, text)
+			self.model.set(iter, C_PERCENT, int(percent))
+			text = self._format_time(eta)
+			text += '\n'
+			#This should make the string Kb/s, 
+			#where 'Kb' part is taken from %s.
+			#Only the 's' after / (which means second) should be translated.
+			text += _('(%(filesize_unit)s/s)') % {'filesize_unit':
+				helpers.convert_bytes(speed)}
+			self.model.set(iter, C_TIME, text)
+			
+			# try to guess what should be the status image
 			if file_props['type'] == 'r':
 				status = 'download'
 			else:
@@ -393,7 +489,7 @@ _('Connection with peer cannot be established.'))
 		session id'''
 		iter = self.model.get_iter_root()
 		while iter:
-			if typ + sid == self.model[iter][4].decode('utf-8'):
+			if typ + sid == self.model[iter][C_SID].decode('utf-8'):
 				return iter
 			iter = self.model.iter_next(iter)
 	
@@ -411,9 +507,17 @@ _('Connection with peer cannot be established.'))
 		properties in it'''
 		file_props = {'file-name' : file_path, 'name' : file_name, 
 			'type' : 's'}
-		if os.path.exists(file_path) and os.path.isfile(file_path):
+		if os.path.isfile(file_path):
+			
 			stat = os.stat(file_path)
-		os.stat(file_path)
+		else:
+			dialogs.ErrorDialog(_('Invalid File'), _('File: ')  + file_path).get_response()
+			return None
+		if stat[6] == 0:
+			dialogs.ErrorDialog(_('Invalid File'), 
+			_('It is not possible to send empty files')).get_response()
+			return None
+		file_props['elapsed-time'] = 0
 		file_props['size'] = unicode(stat[6])
 		file_props['sid'] = self.get_sid()
 		file_props['completed'] = False
@@ -428,6 +532,7 @@ _('Connection with peer cannot be established.'))
 		self.on_transfers_list_leave_notify_event(None)
 		if file_props is None:
 			return
+		file_props['elapsed-time'] = 0
 		self.files_props[file_props['type']][file_props['sid']] = file_props
 		iter = self.model.append()
 		text_labels = '<b>' + _('Name: ') + '</b>\n' 
@@ -442,7 +547,7 @@ _('Connection with peer cannot be established.'))
 			file_name = file_props['name']
 		text_props = gtkgui_helpers.escape_for_pango_markup(file_name) + '\n'
 		text_props += gtkgui_helpers.escape_for_pango_markup(contact.name)
-		self.model.set(iter, 1, text_labels, 2, text_props, 4, \
+		self.model.set(iter, 1, text_labels, 2, text_props, C_SID,
 			file_props['type'] + file_props['sid'])
 		self.set_progress(file_props['type'], file_props['sid'], 0, iter)
 		if file_props.has_key('started') and file_props['started'] is False:
@@ -453,6 +558,7 @@ _('Connection with peer cannot be established.'))
 			status = 'upload'
 		file_props['tt_account'] = account
 		self.set_status(file_props['type'], file_props['sid'], status)
+		self.set_cleanup_sensitivity()
 		self.window.show_all()
 	
 	def on_transfers_list_motion_notify_event(self, widget, event):
@@ -471,7 +577,7 @@ _('Connection with peer cannot be established.'))
 			except:
 				self.tooltip.hide_tooltip()
 				return
-			sid = self.model[iter][4].decode('utf-8')
+			sid = self.model[iter][C_SID].decode('utf-8')
 			file_props = self.files_props[sid[0]][sid[1:]]
 			if file_props is not None:
 				if self.tooltip.timeout == 0 or self.tooltip.id != props[0]:
@@ -520,21 +626,30 @@ _('Connection with peer cannot be established.'))
 			return True
 		if file_props.has_key('completed') and file_props['completed']:
 			return True
-		if not file_props.has_key('stopped') or not \
-			file_props['stopped']:
+		if file_props.has_key('connected') and file_props['connected'] == False:
+			return True
+		if not file_props.has_key('stopped') or not file_props['stopped']:
 			return False
 		return True
-		
+
+	def set_cleanup_sensitivity(self):
+		''' check if there are transfer rows and set cleanup_button 
+		sensitive, or insensitive if model is empty'''
+		if len(self.model) == 0:
+			self.cleanup_button.set_sensitive(False)
+		else:
+			self.cleanup_button.set_sensitive(True)
+	
 	def set_all_insensitive(self):
 		''' make all buttons/menuitems insensitive '''
 		self.pause_button.set_sensitive(False)
 		self.pause_menuitem.set_sensitive(False)
 		self.continue_menuitem.set_sensitive(False)
-		self.remove_button.set_sensitive(False)
 		self.remove_menuitem.set_sensitive(False)
 		self.cancel_button.set_sensitive(False)
 		self.cancel_menuitem.set_sensitive(False)
 		self.open_folder_menuitem.set_sensitive(False)
+		self.set_cleanup_sensitivity()
 	
 	def set_buttons_sensitive(self, path, is_row_selected):
 		''' make buttons/menuitems sensitive as appropriate to 
@@ -543,9 +658,8 @@ _('Connection with peer cannot be established.'))
 			self.set_all_insensitive()
 			return
 		current_iter = self.model.get_iter(path)
-		sid = self.model[current_iter][4].decode('utf-8')
+		sid = self.model[current_iter][C_SID].decode('utf-8')
 		file_props = self.files_props[sid[0]][sid[1:]]
-		self.remove_button.set_sensitive(is_row_selected)
 		self.remove_menuitem.set_sensitive(is_row_selected)
 		self.open_folder_menuitem.set_sensitive(is_row_selected)
 		is_stopped = False
@@ -594,33 +708,27 @@ _('Connection with peer cannot be established.'))
 			if selected_path == path:
 				is_selected = True
 		self.set_buttons_sensitive(path, is_selected)
+		self.set_cleanup_sensitivity()
 		return True
 	
-	def on_remove_button_clicked(self, widget):
-		selected = self.tree.get_selection().get_selected()
-		if selected is None or selected[1] is None:
-			return 
-		s_iter = selected[1]
-		sid = self.model[s_iter][4].decode('utf-8')
-		file_props = self.files_props[sid[0]][sid[1:]]
-		if not file_props.has_key('tt_account'):
-			# file transfer is not set yet
-			return 
-		account = file_props['tt_account']
-		if not gajim.connections.has_key(account):
-			# no connection to the account
-			return
-		gajim.connections[account].remove_transfer(file_props)
-		self.model.remove(s_iter)
+	def on_cleanup_button_clicked(self, widget):
+		i = len(self.model) - 1
+		while i >= 0:
+			iter = self.model.get_iter((i))
+			sid = self.model[iter][C_SID].decode('utf-8')
+			file_props = self.files_props[sid[0]][sid[1:]]
+			if self.is_transfer_stoped(file_props):
+				self._remove_transfer(iter, sid, file_props)
+			i -= 1
+		self.tree.get_selection().unselect_all()
 		self.set_all_insensitive()
 	
 	def toggle_pause_continue(self, status):
 		if status:
 			label = _('Pause')
 			self.pause_button.set_label(label)
-			if gtk.gtk_version >= (2, 6, 0) and gtk.pygtk_version >= (2, 6, 0):
-				self.pause_button.set_image(gtk.image_new_from_stock(
-			gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_MENU))
+			self.pause_button.set_image(gtk.image_new_from_stock(
+				gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_MENU))
 			
 			self.pause_menuitem.set_sensitive(True)
 			self.pause_menuitem.set_no_show_all(False)
@@ -630,9 +738,8 @@ _('Connection with peer cannot be established.'))
 		else:
 			label = _('_Continue')
 			self.pause_button.set_label(label)
-			if gtk.gtk_version >= (2, 6, 0) and gtk.pygtk_version >= (2, 6, 0):
-				self.pause_button.set_image(gtk.image_new_from_stock(
-			gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_MENU))
+			self.pause_button.set_image(gtk.image_new_from_stock(
+				gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_MENU))
 			self.pause_menuitem.hide()
 			self.pause_menuitem.set_no_show_all(True)
 			self.continue_menuitem.set_sensitive(True)
@@ -643,9 +750,10 @@ _('Connection with peer cannot be established.'))
 		if selected is None or selected[1] is None:
 			return 
 		s_iter = selected[1]
-		sid = self.model[s_iter][4].decode('utf-8')
+		sid = self.model[s_iter][C_SID].decode('utf-8')
 		file_props = self.files_props[sid[0]][sid[1:]]
 		if self.is_transfer_paused(file_props):
+			file_props['last-time'] = time.time()
 			file_props['paused'] = False
 			types = {'r' : 'download', 's' : 'upload'}
 			self.set_status(file_props['type'], file_props['sid'], types[sid[0]])
@@ -660,7 +768,7 @@ _('Connection with peer cannot be established.'))
 		if selected is None or selected[1] is None:
 			return 
 		s_iter = selected[1]
-		sid = self.model[s_iter][4].decode('utf-8')
+		sid = self.model[s_iter][C_SID].decode('utf-8')
 		file_props = self.files_props[sid[0]][sid[1:]]
 		if not file_props.has_key('tt_account'):
 			return 
@@ -681,12 +789,12 @@ _('Connection with peer cannot be established.'))
 			# check if the current pointer is at the same path
 			# as it was before setting the timeout
 			iter = self.model.get_iter(props[0])
-			sid = self.model[iter][4].decode('utf-8')
+			sid = self.model[iter][C_SID].decode('utf-8')
 			file_props = self.files_props[sid[0]][sid[1:]]
 			rect =  self.tree.get_cell_area(props[0],props[1])
 			position = widget.window.get_origin()
 			self.tooltip.show_tooltip(file_props , (pointer[0], rect.height ), 
-				 (position[0], position[1] + rect.y + self.height_diff))
+				(position[0], position[1] + rect.y + self.height_diff))
 		else:
 			self.tooltip.hide_tooltip()
 	
@@ -704,10 +812,6 @@ _('Connection with peer cannot be established.'))
 
 	def show_context_menu(self, event, iter):
 		# change the sensitive propery of the buttons and menuitems
-		if len(self.model) == 0:
-			self.clean_up_menuitem.set_sensitive(False)
-		else:
-			self.clean_up_menuitem.set_sensitive(True)
 		path = None
 		if iter is not None:
 			path = self.model.get_path(iter)
@@ -776,27 +880,12 @@ _('Connection with peer cannot be established.'))
 			if path is not None:
 				return True
 		
-	
-	def on_clean_up_menuitem_activate(self, widget):
-		i = len(self.model) - 1
-		while i >= 0:
-			iter = self.model.get_iter((i))
-			sid = self.model[iter][4].decode('utf-8')
-			file_props = self.files_props[sid[0]][sid[1:]]
-			if file_props.has_key('completed') and file_props['completed']:
-				self.model.remove(iter)
-			elif file_props.has_key('stopped') and file_props['stopped']:
-				self.model.remove(iter)
-			i -= 1
-		self.tree.get_selection().unselect_all()
-		self.set_all_insensitive()
-		
 	def on_open_folder_menuitem_activate(self, widget):
 		selected = self.tree.get_selection().get_selected()
 		if selected is None or selected[1] is None:
 			return 
 		s_iter = selected[1]
-		sid = self.model[s_iter][4].decode('utf-8')
+		sid = self.model[s_iter][C_SID].decode('utf-8')
 		file_props = self.files_props[sid[0]][sid[1:]]
 		if not file_props.has_key('file-name'):
 			return
@@ -812,10 +901,16 @@ _('Connection with peer cannot be established.'))
 	
 	def on_pause_menuitem_activate(self, widget):
 		self.on_pause_restore_button_clicked(widget)
-		#FIXME: change the stock
 		
 	def on_remove_menuitem_activate(self, widget):
-		self.on_remove_button_clicked(widget)
+		selected = self.tree.get_selection().get_selected()
+		if selected is None or selected[1] is None:
+			return 
+		s_iter = selected[1]
+		sid = self.model[s_iter][C_SID].decode('utf-8')
+		file_props = self.files_props[sid[0]][sid[1:]]
+		self._remove_transfer(s_iter, sid, file_props)
+		self.set_all_insensitive()
 
 	def on_file_transfers_window_key_press_event(self, widget, event):
 		if event.keyval == gtk.keysyms.Escape: # ESCAPE
