@@ -1,11 +1,17 @@
 ##	vcard.py (has VcardWindow class)
 ##
-## Gajim Team:
+## Contributors for this file:
 ##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Vincent Hanquez <tab@snarc.org>
 ##	- Nikos Kouremenos <kourem@gmail.com>
 ##
-##	Copyright (C) 2003-2005 Gajim Team
+## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
+##                         Vincent Hanquez <tab@snarc.org>
+## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
+##                    Vincent Hanquez <tab@snarc.org>
+##                    Nikos Kouremenos <nkour@jabber.org>
+##                    Dimitur Kirov <dkirov@gmail.com>
+##                    Travis Shirk <travis@pobox.com>
+##                    Norman Rasmussen <norman@rasmussen.co.za>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -25,6 +31,7 @@ import base64
 import mimetypes
 import os
 import sys
+import gtkgui_helpers
 import dialogs
 
 from common import helpers
@@ -38,10 +45,39 @@ gtk.glade.textdomain (APP)
 
 GTKGUI_GLADE = 'gtkgui.glade'
 
+def get_avatar_pixbuf_encoded_mime(photo):
+	'''return the pixbuf of the image
+	photo is a dictionary containing PHOTO information'''
+	if not isinstance(photo, dict):
+		return None, None, None
+	img_decoded = None
+	avatar_encoded = None
+	avatar_mime_type = None
+	if photo.has_key('BINVAL') and photo.has_key('TYPE'):
+		img_encoded = photo['BINVAL']
+		avatar_encoded = img_encoded
+		avatar_mime_type = photo['TYPE']
+		try:
+			img_decoded = base64.decodestring(img_encoded)
+		except:
+			pass
+	elif photo.has_key('EXTVAL'):
+		url = photo['EXTVAL']
+		try:
+			fd = urllib.urlopen(url)
+			img_decoded = fd.read()
+		except:
+			pass
+	if img_decoded:
+		pixbuf = gtkgui_helpers.get_pixbuf_from_data(img_decoded)
+	else:
+		pixbuf = None
+	return pixbuf, avatar_encoded, avatar_mime_type
+
 class VcardWindow:
 	'''Class for contact's information window'''
 
-	def __init__(self, contact, plugin, account, vcard = False):
+	def __init__(self, contact, account, vcard = False):
 		#the contact variable is the jid if vcard is true
 		self.xml = gtk.glade.XML(GTKGUI_GLADE, 'vcard_information_window', APP)
 		self.window = self.xml.get_widget('vcard_information_window')
@@ -52,7 +88,6 @@ class VcardWindow:
 		self.publish_button.set_no_show_all(True)
 		self.retrieve_button.set_no_show_all(True)
 		
-		self.plugin = plugin
 		self.contact = contact #don't use it if vcard is true
 		self.account = account
 		self.vcard = vcard
@@ -72,13 +107,29 @@ class VcardWindow:
 		self.xml.signal_autoconnect(self)
 		self.window.show_all()
 
-	def on_vcard_information_window_destroy(self, widget = None):
-		del self.plugin.windows[self.account]['infos'][self.jid]
+	def on_vcard_information_window_destroy(self, widget):
+		del gajim.interface.instances[self.account]['infos'][self.jid]
 
 	def on_vcard_information_window_key_press_event(self, widget, event):
 		if event.keyval == gtk.keysyms.Escape:
 			self.window.destroy()
 
+	def on_log_history_checkbutton_toggled(self, widget):
+		#log conversation history?
+		oldlog = True
+		no_log_for = gajim.config.get_per('accounts', self.account,
+			'no_log_for').split()
+		if self.contact.jid in no_log_for:
+			oldlog = False
+		log = widget.get_active()
+		if not log and not self.contact.jid in no_log_for:
+			no_log_for.append(self.contact.jid)
+		if log and self.contact.jid in no_log_for:
+			no_log_for.remove(self.contact.jid)
+		if oldlog != log:
+			gajim.config.set_per('accounts', self.account, 'no_log_for',
+				' '.join(no_log_for))
+	
 	def on_close_button_clicked(self, widget):
 		'''Save contact information and update the roster on the Jabber server'''
 		if self.vcard:
@@ -89,24 +140,11 @@ class VcardWindow:
 		new_name = name_entry.get_text().decode('utf-8')
 		if new_name != self.contact.name and new_name != '':
 			self.contact.name = new_name
-			for i in self.plugin.roster.get_contact_iter(self.contact.jid, self.account):
-				self.plugin.roster.tree.get_model().set_value(i, 1, new_name)
+			for i in gajim.interface.roster.get_contact_iter(self.contact.jid,
+				self.account):
+				gajim.interface.roster.tree.get_model().set_value(i, 1, new_name)
 			gajim.connections[self.account].update_contact(self.contact.jid,
 				self.contact.name, self.contact.groups)
-		#log history ?
-		oldlog = True
-		no_log_for = gajim.config.get_per('accounts', self.account,
-			'no_log_for').split()
-		if self.contact.jid in no_log_for:
-			oldlog = False
-		log = self.xml.get_widget('log_checkbutton').get_active()
-		if not log and not self.contact.jid in no_log_for:
-			no_log_for.append(self.contact.jid)
-		if log and self.contact.jid in no_log_for:
-			no_log_for.remove(self.contact.jid)
-		if oldlog != log:
-			gajim.config.set_per('accounts', self.account, 'no_log_for',
-				' '.join(no_log_for))
 		self.window.destroy()
 
 	def on_clear_button_clicked(self, widget):
@@ -149,45 +187,44 @@ class VcardWindow:
 		filtr.add_mime_type('image/png')
 		filtr.add_mime_type('image/jpeg')
 		filtr.add_mime_type('image/gif')
-		filtr.add_pattern('*.png')
-		filtr.add_pattern('*.jpg')
-		filtr.add_pattern('*.gif')
-		filtr.add_pattern('*.tif')
-		filtr.add_pattern('*.xpm')
+		filtr.add_mime_type('image/tiff')
+		filtr.add_mime_type('image/x-xpixmap') # xpm
 		dialog.add_filter(filtr)
 		dialog.set_filter(filtr)
 		dialog.set_use_preview_label(False)
 		dialog.set_preview_widget(gtk.Image())
 		dialog.connect('selection-changed', self.update_preview)
 
-		ok = False
-		while not ok:
+		done = False
+		while not done:
 			response = dialog.run()
 			if response == gtk.RESPONSE_OK:
 				f = dialog.get_filename()
-				f = f.decode(sys.getfilesystemencoding())
+				try:
+					f = f.decode(sys.getfilesystemencoding())
+				except UnicodeError:
+					pass
+				else:
+					filesize = os.path.getsize(f) # in bytes
+					if filesize > 32768: # 32 kb
+						dialogs.ErrorDialog(_('The filesize of image "%s" is too large')\
+							% os.path.basename(f),
+						_('The file must not be more than 32 kilobytes.')).get_response()
+						continue
 				if self.image_is_ok(f):
-					ok = True
-			else:
-				ok = True
+					done = True
+			else: # Cancel or WM X button
+				done = True
 		dialog.destroy()
 
-		if f:
-			filesize = os.path.getsize(f) # in bytes
-			if filesize > 8192: # 8 kb
-				dialogs.ErrorDialog(_('The filesize of image "%s" is too large')\
-					% f,
-					_('The file must not be more than 8 kilobytes.')).get_response()
-				return
+		if response == gtk.RESPONSE_OK:
 			fd = open(f, 'rb')
 			data = fd.read()
-			pixbufloader = gtk.gdk.PixbufLoader()
-			pixbufloader.write(data)
-			pixbufloader.close()
-			pixbuf = pixbufloader.get_pixbuf()
+			pixbuf = gtkgui_helpers.get_pixbuf_from_data(data)
 			image = self.xml.get_widget('PHOTO_image')
 			image.set_from_pixbuf(pixbuf)
 			self.avatar_encoded = base64.encodestring(data)
+			# returns None if unknown type
 			self.avatar_mime_type = mimetypes.guess_type(f)[0]
 
 	def set_value(self, entry_name, value):
@@ -199,31 +236,10 @@ class VcardWindow:
 	def set_values(self, vcard):
 		for i in vcard.keys():
 			if i == 'PHOTO':
-				if not isinstance(vcard[i], dict):
-					continue
-				img_decoded = None
-				if vcard[i].has_key('BINVAL') and vcard[i].has_key('TYPE'):
-					img_encoded = vcard[i]['BINVAL']
-					self.avatar_encoded = img_encoded
-					self.avatar_mime_type = vcard[i]['TYPE']
-					try:
-						img_decoded = base64.decodestring(img_encoded)
-					except:
-						pass
-				elif vcard[i].has_key('EXTVAL'):
-					url = vcard[i]['EXTVAL']
-					try:
-						fd = urllib.urlopen(url)
-						img_decoded = fd.read()
-					except:
-						pass
-				if img_decoded:
-					pixbufloader = gtk.gdk.PixbufLoader()
-					pixbufloader.write(img_decoded)
-					pixbufloader.close()
-					pixbuf = pixbufloader.get_pixbuf()
-					image = self.xml.get_widget('PHOTO_image')
-					image.set_from_pixbuf(pixbuf)
+				pixbuf, self.avatar_encoded, self.avatar_mime_type = \
+					get_avatar_pixbuf_encoded_mime(vcard[i])
+				image = self.xml.get_widget('PHOTO_image')
+				image.set_from_pixbuf(pixbuf)
 				continue
 			if i == 'ADR' or i == 'TEL' or i == 'EMAIL':
 				for entry in vcard[i]:
@@ -266,20 +282,38 @@ class VcardWindow:
 		self.xml.get_widget('os_label').set_text(os)
 
 	def fill_jabber_page(self):
+		tooltips = gtk.Tooltips()
 		self.xml.get_widget('nickname_label').set_text(self.contact.name)
 		self.xml.get_widget('jid_label').set_text(self.contact.jid)
 		uf_sub = helpers.get_uf_sub(self.contact.sub)
 		self.xml.get_widget('subscription_label').set_text(uf_sub)
+		eb = self.xml.get_widget('subscription_label_eventbox')
+		if self.contact.sub == 'from':
+			tt_text = _("This contact is interested in your presence information, but you are not interested in his/her presence")
+		elif self.contact.sub == 'to':
+			tt_text = _("You are interested in the contact's presence information, but he/she is not interested in yours")
+		elif self.contact.sub == 'both':
+			tt_text = _("You and the contact are interested in each other's presence information")
+		else: # None
+			tt_text = _("You are not interested in the contact's presence, and neither he/she is interested in yours")
+		tooltips.set_tip(eb, tt_text)
+
 		label = self.xml.get_widget('ask_label')
-		
 		uf_ask = helpers.get_uf_ask(self.contact.ask)
 		label.set_text(uf_ask)
+		eb = self.xml.get_widget('ask_label_eventbox')
+		if self.contact.ask == 'subscribe':
+			tooltips.set_tip(eb,
+			_("You are waiting contact's answer about your subscription request"))
 		self.xml.get_widget('nickname_entry').set_text(self.contact.name)
-		log = 1
+		log = True
 		if self.contact.jid in gajim.config.get_per('accounts', self.account,
 			'no_log_for').split(' '):
-			log = 0
-		self.xml.get_widget('log_checkbutton').set_active(log)
+			log = False
+		checkbutton = self.xml.get_widget('log_history_checkbutton')
+		checkbutton.set_active(log)
+		checkbutton.connect('toggled', self.on_log_history_checkbutton_toggled)
+		
 		resources = '%s (%s)' % (self.contact.resource, unicode(
 			self.contact.priority))
 		uf_resources = self.contact.resource + _(' resource with priority ')\
@@ -317,14 +351,11 @@ class VcardWindow:
 			'resource_prio_label_eventbox')
 		tip.set_tip(resource_prio_label_eventbox, uf_resources)
 		
+		tip = gtk.Tooltips()
+		status_label_eventbox = self.xml.get_widget('status_label_eventbox')
+		tip.set_tip(status_label_eventbox, stats)
 		status_label = self.xml.get_widget('status_label')
-		#FIXME: when gtk2.4 is OOOOLD do it via glade2.10+
-		if gtk.pygtk_version >= (2, 6, 0) and gtk.gtk_version >= (2, 6, 0):
-			tip = gtk.Tooltips()
-			status_label_eventbox = self.xml.get_widget('status_label_eventbox')
-			tip.set_tip(status_label_eventbox, stats)
-			status_label.set_max_width_chars(15)
-		
+		status_label.set_max_width_chars(15)
 		status_label.set_text(stats)
 		
 		gajim.connections[self.account].request_vcard(self.contact.jid)
@@ -379,14 +410,16 @@ class VcardWindow:
 
 		# Avatar
 		if self.avatar_encoded:
-			vcard['PHOTO'] = {'TYPE': self.avatar_mime_type,
-				'BINVAL': self.avatar_encoded}
+			vcard['PHOTO'] = {'BINVAL': self.avatar_encoded}
+			if self.avatar_mime_type:
+				vcard['PHOTO']['TYPE'] = self.avatar_mime_type
 		return vcard
 
 	def on_publish_button_clicked(self, widget):
 		if gajim.connections[self.account].connected < 2:
-			ErrorDialog(_('You are not connected to the server'),
-                    _('Without a connection you can not publish your contact information.')).get_response()
+			dialogs.ErrorDialog(_('You are not connected to the server'),
+        		_('Without a connection you can not publish your contact '
+        			'information.')).get_response()
 			return
 		vcard = self.make_vcard()
 		nick = ''
@@ -418,6 +451,7 @@ class VcardWindow:
 
 	def change_to_vcard(self):
 		self.xml.get_widget('information_notebook').remove_page(0)
+		#FIXME: make this string translatable for .10 [thanks Stian]
 		self.xml.get_widget('nickname_label').set_text('Personal details')
 		
 		self.publish_button.show()
